import { _decorator, Component, Node, Prefab, instantiate, CCInteger,LabelComponent } from "cc";
import {PlayerController} from "./PlayerController"
const { ccclass, property } = _decorator;

enum BlockType{
    BT_NONE,
    BT_STONE,
};

enum GameState{
    GS_INIT,
    GS_PLAYING,
    GS_END,
};

@ccclass("GameManager")
export class GameManager extends Component {
    /* class member could be defined like this */
    // dummy = '';

    /* use `property` decorator if your want the member to be serializable */
    // @property
    // serializableDummy = 0;
    @property({type:Prefab})
    cubePrfb:Prefab = null;
    
    @property({type: CCInteger})
    public roadLength: Number = 50;

    @property({type: PlayerController})
    public playerCtrl: PlayerController = null;

    @property({type: Node})
    public startMenu: Node = null;

    @property({type: LabelComponent})
    public stepsLabel: LabelComponent = null;

    @property({type: LabelComponent})
    public leaveTimeLbl: LabelComponent = null;

    private _road: number[] = [];

    private _beforeMoveIdx: number = 0;

    private _curLeaveTime: number = -1;

    @property({type:CCInteger})
    public jumpIntervel: number = 5;

    private _curState: GameState = GameState.GS_INIT;
    set curState (value: GameState) {
        switch(value) {
            case GameState.GS_INIT:
                this.init();
                break;
            case GameState.GS_PLAYING:
                this.startMenu.active = false;
                this.stepsLabel.string = '0';   // 将步数重置为0
                this.leaveTimeLbl.string = '0';
                setTimeout(() => {      //直接设置active会直接开始监听鼠标事件，做了一下延迟处理
                    this.playerCtrl.setInputActive(true);
                }, 0.1);
                break;
            case GameState.GS_END:
                break;
        }
        this._curState = value;
    }

    start () {
        this.curState = GameState.GS_INIT;
        this.playerCtrl.node.on('JumpEnd', this.onPlayerJumpEnd, this);
        this.playerCtrl.node.on('JumpStart', this.onPlayerJumpStart, this);
    }

    init () {
        this.startMenu.active = true;
        this.generateRoad();
        this.playerCtrl.setInputActive(false);
        this.playerCtrl.node.setPosition(cc.v3());
        this.playerCtrl.reset();
        this._curLeaveTime = -1;
        this._beforeMoveIdx = 0;
        this.leaveTimeLbl.string = '0';
    }

    onStartButtonClicked() {
        this.curState = GameState.GS_PLAYING;
    }

    onPlayerJumpEnd(moveIndex: number) {
        this.stepsLabel.string = '' + moveIndex;
        this.checkResult(moveIndex);
    }

    onPlayerJumpStart() {
        this._curLeaveTime = -1;
    }

    checkResult(moveIndex: number) {
        if (moveIndex <= this._road.length) {
            if (this._road[moveIndex] == BlockType.BT_NONE) {   //跳到了空方块上
                this.curState = GameState.GS_INIT;
                this.leaveTimeLbl.string = '0';
            } else {
                this._curLeaveTime = this.jumpIntervel;
                this.leaveTimeLbl.string = '' + this.jumpIntervel;
                this.addRoad(moveIndex - this._beforeMoveIdx, moveIndex);
            }
            this._beforeMoveIdx = moveIndex;
        } else {    // 跳过了最大长度
            this.curState = GameState.GS_INIT;
        }
    }

    /**
     * 每跳一次后如果没失败，则加一个跑道并删除旧跑道
     * @param num 
     * @param moveIndex 
     */
    addRoad(num: number, moveIndex:number) {
        cc.log("begin add " + num + " stone")
        for(let j = 0; j < num; j++) {
            let idx = this._road.length;
            if (this._road[idx-1] === BlockType.BT_NONE) {
                this._road.push(BlockType.BT_STONE);
            } else {
                this._road.push(Math.floor(Math.random() * 2));
            }
            let block: Node = this.spawnBlockByType(this._road[idx]);
            if (block) {
                this.node.addChild(block);
                block.setPosition(idx, -1.5, 0);
                cc.log("add stone, idx:" + idx)
            }
        }
        
        if(moveIndex > 3) {
            let p = this.node.children[0];
            p.removeFromParent(true);
            // p.destroy();
        }
    }

    generateRoad() {

        this.node.removeAllChildren(true);

        this._road = [];
        // startPos
        this._road.push(BlockType.BT_STONE);

        for (let i = 1; i < this.roadLength; i++) {
            if (this._road[i-1] === BlockType.BT_NONE) {
                this._road.push(BlockType.BT_STONE);
            } else {
                this._road.push(Math.floor(Math.random() * 2));
            }
        }

        for (let j = 0; j < this._road.length; j++) {
            let block: Node = this.spawnBlockByType(this._road[j]);
            if (block) {
                block.setPosition(j, -1.5, 0);
                this.node.addChild(block);
            }
        }
    }

    spawnBlockByType(type: BlockType) {
        let block = null;
        switch(type) {
            case BlockType.BT_STONE:
                block = instantiate(this.cubePrfb);
                break;
        }

        return block;
    }

    update (deltaTime: number) {
        // Your update function goes here.
        if(this._curLeaveTime > 0) {
            this._curLeaveTime -= deltaTime;
            if(this._curLeaveTime <= 0) {
                this.curState = GameState.GS_INIT;
            } else {
                this.leaveTimeLbl.string = '' + this._curLeaveTime.toFixed(1);
            }
        }
    }
}
